import os 
from __future__ import print_function
import argparse
import time

import sys
import os
import glob
import argparse
import dill
import json

parser = argparse.ArgumentParser(description='Downloader for WMLA')
parser.add_argument('--data_dir', type=str, default="/gfps/data_dir", metavar='N',
                    help='input data directory on the WMLA shared filesystem')
parser.add_argument('--result_dir', type=str, default="/gfps/result_dir", metavar='N',
                    help='input result directory on the WMLA shared filesystem')
args = parser.parse_args()
print(args)

DATA_DIR = os.getenv("DATA_DIR")
RESULT_DIR = os.getenv("RESULT_DIR")

if __name__ == "__main__":

    with open("./func.pickle", "rb") as f: 
            func = dill.load(f)

    with open("./params.json", "r") as f:
            kwargs = json.load( f)

    download_dir = DATA_DIR + "/" + args.data_dir 

    kwargs["download_dir"] = download_dir

    func(**kwargs)





# from ibm_wmla.wmla_api_v1 import DeepLearningImpactV1
from ibm_wmla.wmla_api_edi_v2 import ElasticDistributedInferenceV2
from ibm_wmla.edi_authentication import ElasticDistributedInferenceAuthenticator
from ibm_cloud_sdk_core.authenticators import BasicAuthenticator
from ibm_cloud_sdk_core.authenticators import BearerTokenAuthenticator


class Connection(object):

    def __init__(self, service_url, service_instance, wmla_v1=False, edi=False, 
                 apikey=None, username=None, password=None, user_access_token = None):
        self.url = service_url
        self.instance = service_instance
        self.apikey = apikey
        self.user = username
        self.password = password
        self.user_access_token = user_access_token
        self.legacy = wmla_v1
        self.edi = edi

    def connect(self):
        if self.legacy:
            if not self.edi:
                print("Connecting to DLI")
                # create the authenticator
                authenticator = BasicAuthenticator(self.user, self.password)
                self.service_dli = DeepLearningImpactV1(
                        authenticator=authenticator)
                self.service_dli.configure_service(self.instance)
                self.service_dli.set_service_url(self.url)
            else:
                print("Connecting to EDI")
                edi_authenticator = ElasticDistributedInferenceAuthenticator(self.user, self.password, self.user_access_token, self.url, disable_ssl_verification = True)
                print("EDI Token created")
                self.service_edi = ElasticDistributedInferenceV2(
                        authenticator=edi_authenticator)
                self.service_edi.configure_service(self.instance)
                self.service_edi.set_service_url(self.url)

                print("EDI Service connected")
        elif self.apikey:
            # in the constructor, assuming control of managing the token
            authenticator = BearerTokenAuthenticator(self.apikey)
            self.service = DeepLearningImpactV1(
                    authenticator=authenticator)
            self.service.configure_service(self.instance)
            self.service.set_service_url(self.url)
        else:
            authenticator = BasicAuthenticator(self.user, self.password)
            self.service = DeepLearningImpactV1(
                    authenticator=authenticator)
            self.service.configure_service(self.instance)
            self.service.set_service_url(self.url)













#!/usr/bin/env python
import redhareapiversion
from redhareapi import Kernel

import json
import os
import sys


## Customer imports
import tensorflow as tf
import vgg19_network
import vgg_preprocessing
import numpy as np


import time
import base64


## Class import
class MiscKernel(Kernel):

    def postprocess(self, preds, **kwargs):
        response_output = preds
        return json.dumps(response_output)

    def attributes_to_dict(self, input_):
        return {att.get("key"):att.get("value") for att in input_.get("attributes")}

    def get_model(self, *args, **kwargs):
        #this is stateful so return true?
        pass

    def setup(self, *args, **kwargs):
        return output


    def load_model(self, *args, **kwargs):
        #this should return the loaded model

        return model

    def preprocces(self, input_data, **kwargs):

        processed_data = input_data

        return processed_data

    def do_inference(processed_data, model, **kwargs):
        preds = model.run(processed_data)

        return preds

    def on_kernel_start(self, kernel_context):
        try:
            self.model_details = json.loads(kernel_context.get_model_description())


            MiscKernel.log_info("on_kernel_start")
            MiscKernel.log_info("mc instance id: " + kernel_context.get_instance_id())
            MiscKernel.log_info("mc input content: " + kernel_context.get_model_description())
            MiscKernel.log_info("Loading Inference Model..")
            MiscKernel.log_info("model_desc" + str(model_desc))

            model_path = model_desc['model_path']
            if model == ' ':
                raise IOError(('error: Model not found.\n'))

            attribute_dict = self.attributes_to_dict(model_desc)

            model = self.get_model(self, model_path, **attribute_dict)
            self.model = self.load_model(self, *model)

            ## Need to output some specifics here
            MiscKernel.log_info("ckpt:" + str(ckpt) +', model:' + model)

            MiscKernel.log_info('loaded network:' + model)

        except Exception as e:
            MiscKernel.log_error("-------------------------")
            MiscKernel.log_error(str(e))
            task_context.set_output_data("Failed due to:" + str(e))
            MiscKernel.log_error("-------------------------")

    def on_task_invoke(self, task_context):
        try:

            outputs = self.setup()

            # Setup batch lists

            while task_context != None:
                MiscKernel.log_debug("on_task_invoke")
                MiscKernel.log_debug("version: " + str(task_context.get_version()))
                MiscKernel.log_debug("model name: " + task_context.get_model_name())
                MiscKernel.log_debug("task id: " + task_context.get_id())
                MiscKernel.log_debug("session id: " + task_context.get_session_id())
                MiscKernel.log_debug("input data: " + task_context.get_input_data())

                # Load input data
                input_data = json.loads(task_context.get_input_data())
                attribute_dict = self.attributes_to_dict(input_data)

                ## Convert attributes to lists or not incase its not batch
                MiscKernel.log_debug("prob_thresh_vec" + str(prob_thresh_vec))
                data_type = input_data['data_type']
                MiscKernel.log_info("data_type: " + data_type)

                # Handle different data types and prep lists for processing
                if data_type == 'image:jpeg_uri':
                    im_files = [os.path.basename(x['value']) for x in input_data['data_value']]
                    im_files_vec.append(im_files)
                    im_files_flat += im_files
                    im_data_value_flat += [x['value'] for x in input_data['data_value']]
                elif data_type == 'image:raw_data':
                    im_files = [x['key'] for x in input_data['data_value']]
                    im_files_vec.append(im_files)
                    im_files_flat += im_files
                    im_data_value_flat += [x['value'] for x in input_data['data_value']]
                else:
                    MiscKernel.log_error("unsupport the data type:" + data_type)


                MiscKernel.log_debug("call next")
                task_context_vec.append(task_context)
                task_context = task_context.next()
                MiscKernel.log_debug("next context: " + str(task_context))

            inputs = []

            ## Pre-process data into the correct format - need exception where the format does not align


            start = time.time()

            ## Loop through the dataset to do predictions
            MiscKernel.log_debug("%.2f%% finished in %.2f s." % (float(idx+batch_size)/len(im_files_flat)*100, (time.time()-start)))

            MiscKernel.log_debug( "Done in %.2f s." % (time.time() - start))


            ## Convert to outputs in the correct format

            MiscKernel.log_debug("return ret "+str(index)+", im_files "+str(im_files)+", ctx "+str(task_context))

            task_context.set_output_data(ret)

        except Exception as e:
            MiscKernel.log_error("-------------------------")
            MiscKernel.log_error( str(e) )
            task_context.set_output_data("Failed due to: " + str(e))
            MiscKernel.log_error("-------------------------")

    def on_kernel_shutdown(self):
        MiscKernel.log_info( 'on_kernel_shutdown')

if __name__ == '__main__':
    obj_kernel = MiscKernel()
    obj_kernel.run()




from ibm_wmla import DeepLearningImpactResTfulApIsV1
from ibm_wmla import DeepLearningImpactResTfulApIsV2
import os
import tempfile
import shutil
from distutils.dir_util import copy_tree



ARGS_V1 = ""
ARGS_V2 = ""

class Training(object):
    def __init__(self, connection, sig_name, dataset, result_dir, framework):
        self.conn = connection.service
        self.sig_name = sig_name
        self.framework = framework
        self.dataset = dataset
        self.result_dir = result_dir


    def train(self, func, libs_path = None, poll_logs=False, **kwargs):
        temp_path = tempfile.mkdtemp()

        with open(temp_path + "/params.json", "w") as f:
            json.dump(kwargs, f)

        if libs_path:
            copy_tree(libs_path, temp_path)

        if isinstance(func, str):
            shutil.copy(func, temp_path + "/train.py")

        else:
            with open(temp_path + "/func.pickle", "wb") as f: 
                dill.dump(func, f)
            
            wmla_asset_path = wmla_client.__file__ + "/assets/train.py"
            shutil.copy(wmla_asset_path, temp_path )

        ##TODO ensure we dont use a GPU for this step
        args_command = "--exec-start %s --model-main train.py --data_dir %s --result_dir %s" % (framework, self.dataset.data_dir, self.result_dir)

        try: 
            response = self.conn.create_exec(self.sig_name, args_command, temp_path + "/train.modelDir.tar" )

            if not response.ok:
                print('submit job failed: code=%s, %s'%(response.status_code, response.get_result()))

            else:
                if poll_logs:
                    utils.query_job_status(response.get_results().get("ID"))
        except:
            print("Failed to submit request")


        return response.get_result()






    def configure(self, framework=None, gpus_per_worker, num_workers):

        pickle.save("/tmp/model_func.pickle", model_train_func)

        args =


        pas


class Training(object):
    def __init__(self, connection):
        self.conn = connection

    def train(self, model_train_func, framework=None, gpus_per_worker, num_workers):
        
        pickle.save("/tmp/model_func.pickle", model_train_func)
        
        args = 


        pas
        
    def _find_modules(self, func):
        
    def _find_module_path(self, module):
        module.__file__ ## should provide the path 
        os.path.dirname(__file__) ## returns abs path 
        
    
    def _create_tar(self):
    
    
    

    def dist_train(self):
        pass

    def elastic_dist_train(self):
        pass

    def hpo(self):
        pass




    # List VPCs
    print("List Templates for the Frameworks")
    try:
        templates = service. get_model_template_details(framework="tensorflow")
        print(templates)
    except ApiException as e:
        print("List Power Cloud failed with status code " + str(e.code) + ": " + e.message)



def update_model_profile_parameters(model_profile, 
                gpu_type, kernel_resource_group, kernel_consumer_path):
    model_profile['kernel']['gpu'] = gpu_type
    model_profile['resource_allocation']['kernel']['resource_group'] = kernel_resource_group
    model_profile['resource_allocation']['kernel']['consumer'] = kernel_consumer_path


###############################################################################
# Licensed Materials - Property of IBM
# 5725-Y38
# @ Copyright IBM Corp. 2018-2020 All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
###############################################################################

import os
import json
import datetime
import traceback

LEVEL = {
            -1: "D",
             0: "I",
             1: "W",
             2: "E"
        }

class KernelContext(object):

    def get_id(self):
        return "local"

    def get_model_description(self):
        #just read the model.json
        with open("./model.json",'r') as load_f:
            model_desc = json.load(load_f)
            if 'model_path' not in model_desc:
                model_desc['model_path'] = os.getcwd()

        return json.dumps(model_desc)


class TaskContext(object):

    def __init__(self, batch=4):
        self.batch_size = batch
    
    def get_id(self):
        return "local"

    def get_uuid(self):
        return "local"

    def get_session_id(self):
        return "local"

    def get_job_id(self):
        return "local"

    def get_input_data(self):
        if self.batch_size:
            self.batch_size -= 1

        #just read the input.json
        with open("./input.json",'r') as load_f:
            return load_f.read()

    def get_input_data_extend(self):
        return self.get_input_data()

    def set_output_data(self, data):
        print("Response data: {}".format(data))
        return True

    def set_output_data_extend(self, data):
        print(data)
        return True

    def report_ready(self, data):
        return True

    def next(self):
        if self.batch_size:
            return self

    def prev(self):
        return None

class Kernel(object):

    def on_kernel_start(self, kernel_context):
        return

    def on_task_invoke(self, task_context):
        return

    def on_kernel_shutdown(self):
        return

    def run(self):
       self.on_kernel_start(KernelContext())
       self.on_task_invoke(TaskContext(5))
       self.on_kernel_shutdown()

    @staticmethod
    def log(level, msg):
        print("{}: {} {}".format(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'), LEVEL[level], msg))

    @staticmethod
    def log_debug(msg):
        Kernel.log(-1, msg)

    @staticmethod
    def log_info(msg):
        Kernel.log(0, msg)

    @staticmethod
    def log_warn(msg):
        Kernel.log(1, msg)

    @staticmethod
    def log_error(msg):
        Kernel.log(2, msg)




#!/usr/bin/env python
import redhareapiversion
from redhareapi import Kernel

import json
import os
import sys
import numpy as np
import tensorflow as tf
import vgg19_network
import vgg_preprocessing
import time
import base64


class Vgg19Kernel(Kernel):
    def _getLabel(self, id, labels = None):
        id = int(id)
        if labels and len(labels) > id:
            return labels[id]
        return str(id)

    def getClassifyInferenceResult(self, sample_id, prediction, prob_thresh = 0.0, label_file = None):
        if label_file and os.path.exists(label_file):
            CLASSES = [line.strip() for line in open(label_file) if line]
        else:
            CLASSES = None
        output_dict = {"type":"classification", "result":[]}

        for i in np.arange(len(prediction)):
            sampleId = os.path.basename(sample_id[i])
            pred = prediction[i]
            top_k = pred.flatten().argsort()[::-1]
            j = 0
            go = True
            while go:
                if (j < len(top_k) and pred[top_k[j]] >= prob_thresh):
                    label = self._getLabel(top_k[j], CLASSES)
                    prob = float('%.3f'%pred[top_k[j]])
                    output_dict['result'].append({"sampleId":sampleId, "label":label, "prob":prob})
                    j+=1
                else:
                    go = False

        return json.dumps(output_dict)

    def get_attribute(self, attributes, attribute_name):
        value = None
        for attribute in attributes:
            Vgg19Kernel.log_debug("get " + attribute_name + " from attribute: "+ str(attribute) )
            if attribute['key'] == attribute_name:
                value = attribute['value']
                break
        return value
    def on_kernel_start(self, kernel_context):
        try:
            Vgg19Kernel.log_info("on_kernel_start")
            Vgg19Kernel.log_info("mc instance id: " + kernel_context.get_instance_id())
            Vgg19Kernel.log_info("mc input content: " + kernel_context.get_model_description())
            model_desc = json.loads(kernel_context.get_model_description())
            Vgg19Kernel.log_info("loading vgg19 inference model..")
            Vgg19Kernel.log_info("model_desc" + str(model_desc))
            model = model_desc['model_path']
            if model == ' ':
                raise IOError(('error: Model not found.\n'))

            IMG_HEIGHT =  int(self.get_attribute(model_desc['attributes'], 'img_height'))
            IMG_WIDTH = int(self.get_attribute(model_desc['attributes'], 'img_width'))
            IMG_DEPTH = int(self.get_attribute(model_desc['attributes'], 'img_depth'))
            MAXIMUM_BATCHSIZE = int(self.get_attribute(model_desc['attributes'], 'maximum_batchsize'))
            NUM_CLASSES =  int(self.get_attribute(model_desc['attributes'],'num_classes'))

            self.num_classes = NUM_CLASSES

            self.label_file = None
            lableFlie=self.get_attribute(model_desc['attributes'],'label_file')
            if lableFlie != None:
                self.label_file = lableFlie
            self.maximum_batchsize = MAXIMUM_BATCHSIZE
            self.image_filename_placeholder = tf.placeholder(tf.string)
            self.image_filecontent = tf.read_file(self.image_filename_placeholder)
            self.input_image = tf.image.decode_image(self.image_filecontent, channels=3)
            self.input_image = vgg_preprocessing.preprocess_image(self.input_image, IMG_HEIGHT, IMG_WIDTH, is_training=False)
            self.image_batch_placeholder = tf.placeholder(tf.float32, [None, IMG_HEIGHT, IMG_WIDTH, IMG_DEPTH])
            logits, _ = vgg19_network.vgg_19(self.image_batch_placeholder, NUM_CLASSES,  is_training=False)
            self.net = tf.nn.softmax(logits)
            self.sess = tf.Session(config=tf.ConfigProto(allow_soft_placement=True))
            self.sess.run(tf.local_variables_initializer())
            saver = tf.train.Saver()

            ckpt = tf.train.get_checkpoint_state(model)
            Vgg19Kernel.log_info("ckpt:" + str(ckpt) +', model:' + model)
            if ckpt and ckpt.model_checkpoint_path:
                Vgg19Kernel.log_info("restoring checkpoint")
                saver.restore(self.sess, ckpt.model_checkpoint_path)

            Vgg19Kernel.log_info('loaded network:' + model)

        except Exception as e:
            Vgg19Kernel.log_error("-------------------------")
            Vgg19Kernel.log_error(str(e))
            task_context.set_output_data("Failed due to:" + str(e))
            Vgg19Kernel.log_error("-------------------------")

    def on_task_invoke(self, task_context):
        try:
            task_context_vec=[]
            im_files_vec=[]
            im_files_flat=[]
            im_data_value_flat=[]
            prob_thresh_vec=[]

            while task_context != None:
                Vgg19Kernel.log_debug("on_task_invoke")
                Vgg19Kernel.log_debug("version: " + str(task_context.get_version()))
                Vgg19Kernel.log_debug("model name: " + task_context.get_model_name())
                Vgg19Kernel.log_debug("task id: " + task_context.get_id())
                Vgg19Kernel.log_debug("session id: " + task_context.get_session_id())
                Vgg19Kernel.log_debug("input data: " + task_context.get_input_data())

                input_data = json.loads(task_context.get_input_data())
                prob_thresh = self.get_attribute(input_data['attributes'], 'threshold')
                if prob_thresh != ' ':
                    prob_thresh_vec.append(float(prob_thresh))
                else:
                    prob_thresh_vec.append(float(0.0))
                Vgg19Kernel.log_debug("prob_thresh_vec" + str(prob_thresh_vec))
                data_type = input_data['data_type']
                Vgg19Kernel.log_info("data_type: " + data_type)
                if data_type == 'image:jpeg_uri':
                    im_files = [os.path.basename(x['value']) for x in input_data['data_value']]
                    im_files_vec.append(im_files)
                    im_files_flat += im_files
                    im_data_value_flat += [x['value'] for x in input_data['data_value']]
                elif data_type == 'image:raw_data':
                    im_files = [x['key'] for x in input_data['data_value']]
                    im_files_vec.append(im_files)
                    im_files_flat += im_files
                    im_data_value_flat += [x['value'] for x in input_data['data_value']]
                else:
                    Vgg19Kernel.log_error("unsupport the data type:" + data_type)
                Vgg19Kernel.log_debug("call next")
                task_context_vec.append(task_context)
                task_context = task_context.next()
                Vgg19Kernel.log_debug("next context: " + str(task_context))

            inputs = []

            if data_type == 'image:jpeg_uri':
                for image_filename in im_data_value_flat:
                    img = self.sess.run(self.input_image, feed_dict={self.image_filename_placeholder: image_filename})
                    inputs.append(img)

            elif data_type == 'image:raw_data':
                for image_file in im_data_value_flat:
                    Vgg19Kernel.log_debug("base64 start decode str len"+ str(len(image_file)))
                    image_content = base64.b64decode(image_file)
                    Vgg19Kernel.log_debug("base64 end decode")
                    img = self.sess.run(self.input_image, feed_dict={self.image_filecontent: image_content})
                    Vgg19Kernel.log_debug("processed image with shape "+str(img.shape))
                    inputs.append(img)
            else:
                Vgg19Kernel.log_error("unsupport the data type:" + data_type)

            inputs = np.array(inputs)
            predictions = np.zeros((len(im_files_flat), self.num_classes), np.float32)
            start = time.time()
            for idx in range(0, len(im_files_flat), self.maximum_batchsize):
                batch_size = min(self.maximum_batchsize, len(im_files_flat) - idx)
                pred = self.sess.run(self.net, feed_dict={self.image_batch_placeholder:inputs[idx:idx+batch_size]})
                predictions[idx:idx+batch_size] = pred
                Vgg19Kernel.log_debug("%.2f%% finished in %.2f s." % (float(idx+batch_size)/len(im_files_flat)*100, (time.time()-start)))

            Vgg19Kernel.log_debug( "Done in %.2f s." % (time.time() - start))

            image_to_pred_dict=dict(zip(im_files_flat, predictions))

            for index in range(len(task_context_vec)):
                task_context=task_context_vec[index]
                im_files=im_files_vec[index]
                Vgg19Kernel.log_debug("return ret "+str(index)+", im_files "+str(im_files)+", ctx "+str(task_context))
                task_imagename=[]
                task_predictions=[]
                for imagename in im_files:
                    pred = image_to_pred_dict[imagename]
                    task_imagename.append(imagename)
                    task_predictions.append(pred)

                prob_thresh = prob_thresh_vec[index]
                ret=self.getClassifyInferenceResult(task_imagename, task_predictions, prob_thresh=prob_thresh, label_file=self.label_file)
                task_context.set_output_data(ret)

        except Exception as e:
            Vgg19Kernel.log_error("-------------------------")
            Vgg19Kernel.log_error( str(e) )
            task_context.set_output_data("Failed due to: " + str(e))
            Vgg19Kernel.log_error("-------------------------")

    def on_kernel_shutdown(self):
        Vgg19Kernel.log_info( 'on_kernel_shutdown')

if __name__ == '__main__':
    obj_kernel = Vgg19Kernel()
    obj_kernel.run()




from ibm_wmla import DeepLearningImpactV1
from ibm_wmla import ElasticDistributedInferenceV2
import os
import tempfile
import shutil
from distutils.dir_util import copy_tree



ARGS_V1 = ""
ARGS_V2 = ""

class Inference(object):
    def __init__(self, connection, model_name):
        self.conn = connection.service_edi
        self.model_name = model_name
    
    def get_model_details(self):
        model_details = self.conn.get_model(self.model_name).result() 
        model_profile = self.conn.get_model_profile(self.model_name).result() 
        return model_details, model_profile

    def get_model_deployed(self):
        ## lift models first and g check lift
        deployed_model_list = self.conn.get_models().result() 
        if self.model_name in deployed_model_list:
            return True
        else:
            return False   

    def _copyfiles(self, paths):
        temp_path = tempfile.mkdtemp()
        for path in list_paths:
            if os.path.isfile(path):
          	    try:
            	    shutil.copy(path, temp_path )
         	    except:
                    print("Failed to copy file")
                    break
            elif os.path.isdir(path):
                try: 
                    copy_tree(path, temp_path)
                except:
                    print("Failed to copy directory")
            else:
                print("Not a file or directory")

    def _generate_files(self, objects):
        
        


    def deploy(self, kernel, model_json, readme, paths):
        if isinstance(kernel, String) or isinstance(kernel, os.Path):
            self._copyfiles()
        elif:
            generate_files
            
            copyfiles
        else:
            error

        tar_files
        if not get_model_deployed():
            try: 
                deploy_responce = sdk.deploy(self.model_name, files)
            except:
                print("failed")
        else:
             print("model deployed please refrlppy instead")

    def start(self):
        if self.get_model_deployed():
            service_edi.start(self.model_name))
        else:
            print("no model named ___ deployed")

    def stop(self):

        
        # step one is create the tar
        #step two is Deploy



    def configure(self, framework=None, gpus_per_worker, num_workers):

        pickle.save("/tmp/model_func.pickle", model_train_func)

        args =


        pas

    def create_model_json(self):
        pass

    def create_kernel(self, get_model_files_func, load_model_func, pre
    process_input_func, inference_func, responce_output_func, clean_up_func):
        # Need a template class that inserts specifics into the kernel.py file


        pass

    def

    def _create_tar(self):




    def dist_train(self):
        pass

    def elastic_dist_train(self):
        pass

    def hpo(self):
        pass




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ibm_wmla_client', 'ibm_wmla_client.assets']

package_data = \
{'': ['*']}

install_requires = \
['ibm-cloud-sdk-core>=3.15.3,<4.0.0',
 'ibm-wmla>=0.1.0,<0.2.0',
 'python_dateutil>=2.5.3,<3.0.0',
 'requests>=2.20,<3.0']

setup_kwargs = {
    'name': 'ibm-wmla-client',
    'version': '0.1.1',
    'description': 'A python based client to simplify using Watson Machine Learning Accelerator',
    'long_description': 'Welcom to ``ibm-wmla-client``\n-------------------------------\n\n``ibm-wmla-client`` is a python based client to simplify using Watson Machine Learning Accelerator Elastic Distributed Inference (WMLA EDI).\n\nPrerequisites\n--------------------\n+ Python 3.5.3 or above\n+ `ibm_wmla`_ 0.1.0 or above\n\n.. _ibm_wmla: https://pypi.org/project/ibm-wmla/\n\n',
    'author': 'Shuang Yu',
    'author_email': 'shuang.yu@ibm.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/sherryyyu/ibm-wmla-python-client',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from redhareapi import Kernel
import json

class InferenceKernel(Kernel):
    def __init__(self, get_model, load_model, preprocess, predict, postprocess, input_structure = None, output_structure = None):
        self.get_model = get_model
        self.load_model = load_model 
        self.preprocess = preprocess ## function
        self.predict = predict
        self.postprocess = postprocess
        self.input_structure = input_structure
        self.output_stucture = output_structure
    
    def install_modules(self):
        pass
    
    def load_modules(self):
        pass

    def get_model(self, model_path, **kwargs):
        return model_path
    
    def load_model(self, model_path, **kwargs):
        ## load_model_code
        pass

    def preprocces(self, data_list, attribute_list, model = None, **kwargs):
        ## preprocessing code here
        pass

    def postprocess(self, predicted_results, attribute_list, **kwargs):
        pass

    def predict_func(self, model, batched_data, batched_attributes, **kwargs):
        pass


    def predict(self, model, preprocessed_data, attribute_list, **kwargs):
        if kwargs.get("batch_size") is not None:
            batch_size = kwargs.get("batch_size")
            data_len = len(preprocessed_data)
            cycles = int(data_len / kwargs.get("batch_size"))
            batched_data = []
            batched_attributes = []
            for a in range(0, cycles):
                batch_min_slice = a*batch_size
                batch_max_slice = (a+1)*batch_size
                batched_data.append(preprocessed_data[batch_min_slice, batch_max_slice])
                batched_attributes.append(attribute_list[batch_min_slice, batch_max_slice])
        else:
            batched_data = preprocessed_data
            batched_attributes = attribute_list
        
        batched_combined = zip(batched_data, batched_attributes)
        for data, attributes in batched_combined:
            predicted_output = self.predict(model, data, attributes, **kwargs)
            postprocessed_output = self.postprocess(predicted_output, attributes, **kwargs)
            self.task_context.set_output_data(json.dumps(output_data))

 
    #def determine_types(dict_):
    #    type_structure = {}
    #    if isinstance(dict_, dict):
    #        if dict_.get("data"):
    #            data_type_lvl1 dict_.get("data")
    #            if isinstance(data_type_lvl1, dict) or isinstance(data_type_lvl1, list):

    #        for k,v in dict_:
                ## Find types 
    
    def _attributes_to_dict(self, input_):
        if input_:
            {att.get("key"):att.get("value") for att in input_.get("attributes")}
        else:
            return input_

    def _add_attributes(self, dict_, task_context):
        dict_["Version"] = task_context.get_version()
        dict_["Model Name"] = task_context.get_model_name()
        dict_["Task ID"] = task_context.get_id()
        dict_["Session ID"] = task_context.get_session_id()
        return dict_

    def _flatten_tasks(self, task_context):
        data_list = []
        attributes_list = []
        while task_context:
            input_data = json.loads(task_context.get_input_data())
            attributes = self._add_attributes(input_data.get("attributes"), task_context)
            data = input_data.get("data")
            data_list.append(data)
            attributes_list.append(attributes)
            task_context.next()
        return data_list, attributes_list

    def on_kernel_start(self, kernel_context):

        InferenceKernel.log_info("Kernel Start")
        InferenceKernel.log_info("Model Instance ID: " + kernel_context.get_instance_id())

        try: 
            self.install_modules()
            self.import_modules()
        except Exception as e:
            InferenceKernel.log_error("-------------------------")
            InferenceKernel.log_error( str(e) )
            task_context.set_output_data("Failed due to: " + str(e))
            InferenceKernel.log_error("-------------------------")

        model_details = json.loads(kernel_context.get_model_description())
        
        InferenceKernel.log_info("Model Input Details: " +  json.dumps(model_details)
        )

        model_path = model_details.get("model_path")
        InferenceKernel.log_info("Model Path Details: " +  model_path)

        self.attributes = self._attributes_to_dict(model_details.get("attributes"))

        InferenceKernel.log_info("Getting Model Path")

        try:
            model_path = self.get_model(model_path, **attributes)
            InferenceKernel.log_info("New Model Path: " + model_path)
        except:
            InferenceKernel.log_error("Failed to get model path")
        try:
            self.model = self.load_model(model_path, **self.attributes)
        except:
            InferenceKernel.log_error("Failed to load model")

    def on_task_invoke(self, task_context):
        try:
            ## Setup required empty lists to store data 
            ## NOTE task context is an array of dicts 
            ## Two ways to handle this: 1 - flatten data into a list and handle list. 2 - is to loop through the array 
            
            ## data is a array of dictionaries 
            ## flatten tast_context 
            # Expect input_data to have the form {"data": <data here>, "attributes": {<attributes>}}
            data_list, attribute_list = self._flatten_tasks(task_context)

            ## Data gets preprocesssed
            preprocessed_data = self.preprocces(data_list, attribute_list, model=self.model, **self.attributes)    
            predicted_results = self.predict(self.model, preprocessed_data, attribute_list, **self.attributes)
        
        except Exception as e:
            InferenceKernel.log_error("-------------------------")
            InferenceKernel.log_error( str(e) )
            task_context.set_output_data("Failed due to: " + str(e))
            InferenceKernel.log_error("-------------------------")


            






from ibm_wmla import DeepLearningImpactResTfulApIsV1
from ibm_wmla import DeepLearningImpactResTfulApIsV2
import os
import tempfile
import shutil
import json
from distutils.dir_util import copy_tree
import dill
import utils
import wmla_client


class Dataset(object):
    def __init__(self, connection, sig_name, data_dir):
        self.conn = connection.service
        self.sig_name = sig_name
        self.data_dir = data_dir         

    def download(self, func, poll_logs=False, **kwargs):
        temp_path = tempfile.mkdtemp()

        with open(temp_path + "/params.json", "w") as f:
            json.dump(kwargs, f)

        if isinstance(func, str):
            shutil.copy(func, temp_path + "/download.py")
        
        else:
            with open(temp_path + "/func.pickle", "wb") as f: 
                dill.dump(func, f)

        wmla_asset_path = wmla_client.__file__ + "/assets/download.py"
        shutil.copy(wmla_asset_path, temp_path )

        ##TODO ensure we dont use a GPU for this step
        args_command = "--exec-start PyTorch --model-main download.py --data_dir %s" % (self.data_dir)

        try: 
            response = self.conn.create_exec(self.sig_name, args_command, temp_path + "/download.modelDir.tar" )

            if not response.ok:
                print('submit job failed: code=%s, %s'%(response.status_code, response.get_result()))

            else:
                if poll_logs:
                    utils.query_job_status(response.get_results().get("ID"))
        except:
            print("Failed to submit request")


        return response.get_result()

    

        







        

        

        

        

        ## dill func to file 
        ## write standard main.py 

        

        # Write func to file 
        # Turn args into json dict
        # package up and submitt to cluster 
        # Get logs 


        


    






